from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase

class TestQueue(ChannelEventsTestCase):

    def test_queue_simple(self):
        """
        Test a simple call through a queue to a single account.
        """
        fixture_file = 'fixtures/queue/queue_simple.json'
        events = self.run_and_get_events(fixture_file)

        caller = SimpleChannel(
            name='SIP/voipgrid-siproute-docker-00000025',
            uniqueid='195176c06ab8-1529939196.518',
            linkedid='195176c06ab8-1529939196.518',
            account_code='15001',
            caller_id=CallerId(num='+31150010001'),
            cid_calling_pres=None,
            connected_line=CallerId(),
            exten='+31150010004',
            state=6,
        )

        target = SimpleChannel(
            name='SIP/150010002-00000026',
            uniqueid='195176c06ab8-1529939198.548',
            linkedid='195176c06ab8-1529939196.518',
            account_code='15001',
            caller_id=CallerId(num='+31150010004'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='+31150010001'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': caller,
                'targets': [target.replace(state=5)],
            }),
            ('on_up', {
                'caller': caller,
                'target': target,
            }),
            ('on_hangup', {
                'caller': caller,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_queue_group(self):
        """
        Test a simple call through a queue to a group.
        """
        fixture_file = 'fixtures/queue/queue_group.json'
        events = self.run_and_get_events(fixture_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-0000001b',
                'targets': [
                    'SIP/150010003-0000001c',
                ],
            }),
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-0000001b',
                'targets': [
                    'SIP/150010002-0000001d',
                ],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-0000001b',
                'target': 'SIP/150010002-0000001d',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-0000001b',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_queue_a_cancel(self):
        """
        Test a call where A exits the queue before B can pick up.
        """
        events = self.run_and_get_events('fixtures/queue/queue_a_cancel.json')
        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000019',
                'targets': [
                    'SIP/150010002-0000001a',
                ],
            }),
            ('on_queue_caller_abandon', {
                'caller': 'SIP/voipgrid-siproute-docker-00000019',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000019',
                'reason': 'completed',
            }),
        ]
        self.assertEqualChannels(expected_events, events)

    def test_queue_attn_xfer(self):
        """
        Test an attended transfer with someone coming through a queue.
        """
        events = self.run_and_get_events('fixtures/queue/queue_attn_xfer.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000087',
                'targets': ['SIP/150010001-00000088'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-00000087',
                'target': 'SIP/150010001-00000088',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000089',
                'targets': ['SIP/150010002-0000008a'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000089',
                'target': 'SIP/150010002-0000008a',
            }),
            ('on_attended_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-00000087',
                'target': 'SIP/150010002-0000008a',
                'transferer': 'SIP/150010001-00000089',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000087',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_queue_blind_xfer(self):
        """
        Test a blind transfer with someone from a queue.
        """
        events = self.run_and_get_events('fixtures/queue/queue_blind_xfer.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-0000008d',
                'targets': ['SIP/150010001-0000008e'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-0000008d',
                'target': 'SIP/150010001-0000008e',
            }),
            ('on_blind_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-0000008d',
                'targets': ['SIP/150010002-0000008f'],
                'transferer': 'SIP/150010001-0000008e',
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-0000008d',
                'target': 'SIP/150010002-0000008f',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-0000008d',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)



from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase


class TestBlondeXfer(ChannelEventsTestCase):
    """
    Test call state notifications for blonde transfers.

    A blonde transfer (also known as semi-attended transfer) is a type of
    transfer which looks like an attended transfer, but the transferer
    doesn't wait for person C to pick up.
    """

    def test_xfer_blonde_abacbc(self):
        """
        Test blonde transfer where A initiates the transfer.
        """
        events_file = 'fixtures/xfer_blonde/xfer_blonde_abacbc.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='SIP/150010001-00000090',
            uniqueid='f29ea68048f6-1530023922.2627',
            linkedid='f29ea68048f6-1530023922.2627',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
            state=6,
        )

        a_chan_transferer = a_chan.replace(
            name='SIP/150010001-00000092',
            uniqueid='f29ea68048f6-1530023931.2654',
            linkedid='f29ea68048f6-1530023931.2654',
            exten='203',
        )

        b_chan = SimpleChannel(
            name='SIP/150010002-00000091',
            uniqueid='f29ea68048f6-1530023922.2635',
            linkedid='f29ea68048f6-1530023922.2627',
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        b_chan_transferring = b_chan.replace(
            exten='203',
        )

        b_chan_transferred = b_chan_transferring.replace(
            connected_line=CallerId(),
        )

        c_chan = SimpleChannel(
            name='SIP/150010003-00000093',
            uniqueid='f29ea68048f6-1530023932.2662',
            linkedid='f29ea68048f6-1530023931.2654',
            account_code='150010001',
            caller_id=CallerId(num='203'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        c_chan_transferred = c_chan.replace(
            connected_line=CallerId(num='202'),
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_b_dial', {
                'caller': a_chan_transferer.replace(state=4),
                'targets': [c_chan.replace(state=5)],
            }),
            ('on_blonde_transfer', {
                'caller': b_chan_transferring,
                'targets': [c_chan.replace(state=5)],
                'transferer': a_chan_transferer.replace(state=4),
            }),
            ('on_up', {
                'caller': b_chan_transferred,
                'target': c_chan_transferred,
            }),
            ('on_hangup', {
                'caller': b_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_xfer_blonde_abbcac(self):
        """
        Test blonde transfer where B initiates the transfer.
        """
        events_file = 'fixtures/xfer_blonde/xfer_blonde_abbcac.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='SIP/voipgrid-siproute-docker-00000096',
            uniqueid='f29ea68048f6-1530024929.2709',
            linkedid='f29ea68048f6-1530024929.2709',
            account_code='15001',
            caller_id=CallerId(num='+31260010001'),
            cid_calling_pres=None,
            connected_line=CallerId(name='', num=''),
            exten='+31150010001',
            state=6,
        )

        a_chan_transferred = a_chan.replace(
            exten='202',
        )

        b_chan = SimpleChannel(
            name='SIP/150010001-00000097',
            uniqueid='f29ea68048f6-1530024929.2716',
            linkedid='f29ea68048f6-1530024929.2709',
            account_code='15001',
            caller_id=CallerId(num='+31150010001'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='+31260010001'),
            exten='s',
            state=6,
        )

        b_chan_transferer = b_chan.replace(
            name='SIP/150010001-00000098',
            uniqueid='f29ea68048f6-1530024939.2753',
            linkedid='f29ea68048f6-1530024939.2753',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
        )

        c_chan = SimpleChannel(
            name='SIP/150010002-00000099',
            uniqueid='f29ea68048f6-1530024939.2761',
            linkedid='f29ea68048f6-1530024939.2753',
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        c_chan_transferred = c_chan.replace(
            connected_line=CallerId(num='+31260010001'),
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_b_dial', {
                'caller': b_chan_transferer.replace(state=4),
                'targets': [c_chan.replace(state=5)],
            }),
            ('on_blonde_transfer', {
                'caller': a_chan_transferred,
                'targets': [c_chan.replace(state=5)],
                'transferer': b_chan_transferer.replace(state=4),
            }),
            ('on_up', {
                'caller': a_chan_transferred,
                'target': c_chan_transferred,
            }),
            ('on_hangup', {
                'caller': a_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_xfer_blonde_reject(self):
        """
        Test blonde transfer where the transfer target rejects the call.
        """
        events_file = 'fixtures/xfer_blonde/xfer_blonde_reject.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a1',
                'targets': ['SIP/150010001-000000a2'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a1',
                'target': 'SIP/150010001-000000a2',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-000000a3',
                'targets': ['SIP/150010002-000000a4'],
            }),
            ('on_blonde_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a1',
                'targets': ['SIP/150010002-000000a4'],
                'transferer': 'SIP/150010001-000000a3',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a1',
                'reason': 'busy',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_xfer_blonde_group_b(self):
        """
        Test blonde transfer where the call is transferred to a group by B.
        """
        events_file = 'fixtures/xfer_blonde/xfer_blonde_group_b.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a7',
                'targets': ['SIP/150010001-000000a8'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a7',
                'target': 'SIP/150010001-000000a8',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-000000a9',
                'targets': [
                    'SIP/150010002-000000aa',
                ],
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-000000a9',
                'targets': [
                    'SIP/150010003-000000ab',
                ],
            }),
            ('on_blonde_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a7',
                'targets': [
                    'SIP/150010002-000000aa',
                    'SIP/150010003-000000ab',
                ],
                'transferer': 'SIP/150010001-000000a9'
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a7',
                'target': 'SIP/150010002-000000aa',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-000000a7',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_xfer_blonde_group_a(self):
        """
        Test blonde transfer where the call is transferred to a group by A.
        """
        events_file = 'fixtures/xfer_blonde/xfer_blonde_group_a.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-000000ac',
                'targets': ['SIP/voipgrid-siproute-docker-000000ad'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-000000ac',
                'target': 'SIP/voipgrid-siproute-docker-000000ad',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-000000b0',
                'targets': [
                    'SIP/150010002-000000b1',
                ],
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-000000b0',
                'targets': [
                    'SIP/150010003-000000b2',
                ],
            }),
            ('on_blonde_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-000000ad',
                'targets': [
                    'SIP/150010002-000000b1',
                    'SIP/150010003-000000b2',
                ],
                'transferer': 'SIP/150010001-000000b0',
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-000000ad',
                'target': 'SIP/150010002-000000b1',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-000000ad',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)



from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase


class TestOriginate(ChannelEventsTestCase):

    def test_ctd_account_account(self):
        """
        Click-to-dial call between a phoneaccount and an internal number.

        In this scenario:
        1. 201 is dialed,
        2. 201 picks up,
        3. 202 is dialed
        4. 202 picks up
        5. 201 hangs up

        Which is reported as:
        1. 201 calls 202
        2. 202 picks up
        3. 201 hangs up
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-account.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000002',
                'targets': ['SIP/150010002-00000003'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000002',
                'target': 'SIP/150010002-00000003',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-00000002',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_ctd_account_world(self):
        """
        Click-to-dial call between a phoneaccount and an external number.

        In this scenario:
        1. 201 is dialed
        2. 201 picks up
        3. +31260010001 is dialed
        4. +31260010001 picks up
        5. +31260010001 hangs up

        Which is reported as:
        1. 201 dials +31260010001
        2. +31260010001 picks up
        3. +31260010001 hangs up
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-world.json')

        calling_chan = SimpleChannel(
            account_code='15001',
            caller_id=CallerId(num='201'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Calling...', num='+31150010001'),
            exten='+31260010001',
            linkedid='c4061ca6474c-1531990515.302',
            name='SIP/150010001-00000008',
            state=6,
            uniqueid='c4061ca6474c-1531990515.317',
        )

        target_chan = SimpleChannel(
            account_code='15001',
            caller_id=CallerId(num='+31260010001'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='+31150010001'),
            exten='s',
            linkedid='c4061ca6474c-1531990515.302',
            name='SIP/voipgrid-siproute-docker-00000009',
            state=6,
            uniqueid='c4061ca6474c-1531990517.363',
        )

        expected_events = [
            ('on_b_dial', {
                'caller': calling_chan,
                'targets': [target_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': calling_chan,
                'target': target_chan,
            }),
            ('on_hangup', {
                'caller': calling_chan,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_ctd_account_world_deny_a(self):
        """
        Click-to-dial call between a phoneaccount and an external number.

        In this scenario:
        1. 201 is dialed
        2. 201 refuses the call

        Which is reported as:
        1. Nothing, because 201 never called anyone.
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-world-deny-a.json')

        self.assertEqual([], events)

    def test_ctd_account_world_deny_b(self):
        """
        Click-to-dial call between a phoneaccount and an external number.

        In this scenario:
        1. 201 is dialed
        2. 201 picks up
        3. +31260010001 is dialed
        4. +31260010001 rejects the call

        Which is reported as:
        1. 201 dials +31260010001
        2. +31260010001 hangs up
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-world-deny-b.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000011',
                'targets': ['SIP/voipgrid-siproute-docker-00000012'],
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-00000011',
                'reason': 'busy',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_ctd_attn_xfer_abbcac(self):
        """
        Click-to-dial with an attended transfer initiated by B.
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-account-xfer-attn-abbcac.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-0000001d',
                'targets': ['SIP/150010002-0000001e'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-0000001d',
                'target': 'SIP/150010002-0000001e',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010002-0000001f',
                'targets': ['SIP/150010003-00000020'],
            }),
            ('on_up', {
                'caller': 'SIP/150010002-0000001f',
                'target': 'SIP/150010003-00000020',
            }),
            ('on_attended_transfer', {
                'caller': 'SIP/150010001-0000001d',
                'target': 'SIP/150010003-00000020',
                'transferer': 'SIP/150010002-0000001f',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-0000001d',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_ctd_attn_xfer_abacbc(self):
        """
        Click-to-dial with an attended transfer initiated by A.
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-world-xfer-attn-abacbc.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000025',
                'targets': ['SIP/voipgrid-siproute-docker-00000026'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000025',
                'target': 'SIP/voipgrid-siproute-docker-00000026',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000029',
                'targets': ['SIP/150010003-0000002a'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000029',
                'target': 'SIP/150010003-0000002a',
            }),
            ('on_attended_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-00000026',
                'target': 'SIP/150010003-0000002a',
                'transferer': 'SIP/150010001-00000029',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000026',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_ctd_account_world_no_ringing(self):
        """
        Click-to-dial where the B side never reaches state 5 RINGING.
        """
        events = self.run_and_get_events(
            'fixtures/originate/ctd-account-world-no-ringing.json')

        calling_chan = SimpleChannel(
            account_code='15001',
            caller_id=CallerId(num='2401'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Calling...', num='+31150010001'),
            exten='+31260010001',
            linkedid='ua5-ams-1552575068.23242646',
            name='SIP/150010063-0015f5f1',
            state=6,
            uniqueid='ua5-ams-1552575068.23242663',
        )

        target_chan = SimpleChannel(
            account_code='15001',
            caller_id=CallerId(num='+31260010001'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='+31150010001'),
            exten='s',
            linkedid='ua5-ams-1552575068.23242646',
            name='SIP/voipgrid-siproute-ams-0015f5f3',
            state=6,
            uniqueid='ua5-ams-1552575069.23242717',
        )

        self.maxDiff = None

        expected_events = [
            ('on_b_dial', {
                'caller': calling_chan,
                'targets': [target_chan],  # no .replace(state=5) here
            }),
            ('on_up', {
                'caller': calling_chan,
                'target': target_chan,
            }),
            ('on_hangup', {
                'caller': calling_chan,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_cmn_world_world(self):
        """
        Call-me-now call between two external numbers.
        """
        events = self.run_and_get_events(
            'fixtures/originate/cmn-world-world.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-0000002b',
                'targets': ['SIP/voipgrid-siproute-docker-0000002e'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-0000002b',
                'target': 'SIP/voipgrid-siproute-docker-0000002e',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-0000002b',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_cmn_world_account_unaccepted(self):
        """
        Call-me-now call between two external numbers where A does not accept.

        +31260010001 is dialed,
        +31260010001 picks up and does not accept.
        """
        events = self.run_and_get_events(
            'fixtures/originate/cmn-world-world-unaccepted.json')

        self.assertEqual([], events)



from collections import Iterable
import os

from cacofonisk import BaseReporter
from cacofonisk import EventHandler
from cacofonisk.channel import SimpleChannel
from cacofonisk.runners.file_runner import FileRunner
from cacofonisk.utils.testcases import BaseTestCase


class TestReporter(BaseReporter):
    """
    TestReporter collects the events generated by EventHandler.

    The events can be retrieved later to see which events have been generated
    during the test.
    """

    def __init__(self):
        super(TestReporter, self).__init__()
        self.events = []

    def on_b_dial(self, caller, targets):
        self.events.append(('on_b_dial', {
            'caller': caller,
            'targets': sorted([target for target in targets]),
        }))

    def on_attended_transfer(self, caller, transferer, target):
        self.events.append(('on_attended_transfer', {
            'caller': caller,
            'transferer': transferer,
            'target': target,
        }))

    def on_blind_transfer(self, caller, transferer, targets):
        self.events.append(('on_blind_transfer', {
            'caller': caller,
            'transferer': transferer,
            'targets': sorted([target for target in targets])
        }))

    def on_blonde_transfer(self, caller, transferer, targets):
        self.events.append(('on_blonde_transfer', {
            'caller': caller,
            'transferer': transferer,
            'targets': sorted([target for target in targets])
        }))

    def on_up(self, caller, target):
        self.events.append(('on_up', {
            'caller': caller,
            'target': target,
        }))

    def on_hangup(self, caller, reason):
        self.events.append(('on_hangup', {
            'caller': caller,
            'reason': reason,
        }))

    def on_queue_caller_abandon(self, caller):
        self.events.append(('on_queue_caller_abandon', {
            'caller': caller,
        }))

class BogoRunner(object):
    def __init__(self, events, reporter):
        self.events = events
        self.reporter = reporter
        self.channel_managers = []

    def run(self):
        handler = EventHandler(reporter=self.reporter)
        interesting_events = handler.event_handlers().keys()
        for event in self.events:
            if (
                    not handler.FILTER_EVENTS or
                    event['Event'] in interesting_events
            ):
                handler.on_event(event)

        self.channel_managers.append(handler)


class ChannelEventsTestCase(BaseTestCase):
    """
    Run event tests based on the JSON sample data.
    """
    def events_from_tuples(cls, tuples):
        """
        Convert a list of tuples to the expected event list.

        Args:
            tuples: An iterable with iterables, see example.

        Returns:
            tuple: A tuple of dictionaries, see example.
        """
        results = []

        for data in tuples:
            event_name, event_data = data
            event_data['event'] = event_name
            results.append(event_data)

        return tuple(results)

    def run_and_get_events(self, filename, reporter=None):
        absolute_path = os.path.join(os.path.dirname(__file__), filename)

        if reporter is None:
            reporter = TestReporter()

        runner = FileRunner([absolute_path], reporter=reporter)
        runner.run()

        assert len(runner.channel_managers) == 1
        return reporter.events

    def _pluck_channel_name_from_event(self, event):
        name, data = event
        data = data.copy()

        for key, value in data.items():
            if isinstance(value, SimpleChannel):
                data[key] = value.name
            elif isinstance(value, Iterable) and not isinstance(value, str):
                data[key] = [item.name for item in value]

        return name, data

    def assertEqualChannels(self, expected, actual):
        actual_channels = [self._pluck_channel_name_from_event(event) for
                           event in actual]

        self.assertEqual(expected, actual_channels)



"""
List the hangup causes of Asterisk.

These are the reasons why an Asterisk channel has ended.
Taken from: https://wiki.asterisk.org/wiki/display/AST/Hangup+Cause+Mappings
"""
AST_CAUSE_UNKNOWN = 0
AST_CAUSE_UNALLOCATED = 1
AST_CAUSE_NO_ROUTE_TRANSIT_NET = 2
AST_CAUSE_NO_ROUTE_DESTINATION = 3
AST_CAUSE_MISDIALLED_TRUNK_PREFIX = 5
AST_CAUSE_CHANNEL_UNACCEPTABLE = 6
AST_CAUSE_CALL_AWARDED_DELIVERED = 7
AST_CAUSE_PRE_EMPTED = 8
AST_CAUSE_NUMBER_PORTED_NOT_HERE = 14
AST_CAUSE_NORMAL_CLEARING = 16
AST_CAUSE_USER_BUSY = 17
AST_CAUSE_NO_USER_RESPONSE = 18
AST_CAUSE_NO_ANSWER = 19
AST_CAUSE_SUBSCRIBER_ABSENT = 20
AST_CAUSE_CALL_REJECTED = 21
AST_CAUSE_NUMBER_CHANGED = 22
AST_CAUSE_REDIRECTED_TO_NEW_DESTINATION = 23
AST_CAUSE_ANSWERED_ELSEWHERE = 26
AST_CAUSE_DESTINATION_OUT_OF_ORDER = 27
AST_CAUSE_INVALID_NUMBER_FORMAT = 28
AST_CAUSE_FACILITY_REJECTED = 29
AST_CAUSE_RESPONSE_TO_STATUS_ENQUIRY = 30
AST_CAUSE_NORMAL_UNSPECIFIED = 31
AST_CAUSE_NORMAL_CIRCUIT_CONGESTION = 34
AST_CAUSE_NETWORK_OUT_OF_ORDER = 38
AST_CAUSE_NORMAL_TEMPORARY_FAILURE = 41
AST_CAUSE_SWITCH_CONGESTION = 42
AST_CAUSE_ACCESS_INFO_DISCARDED = 43
AST_CAUSE_REQUESTED_CHAN_UNAVAIL = 44
AST_CAUSE_FACILITY_NOT_SUBSCRIBED = 50
AST_CAUSE_OUTGOING_CALL_BARRED = 52
AST_CAUSE_INCOMING_CALL_BARRED = 54
AST_CAUSE_BEARERCAPABILITY_NOTAUTH = 57
AST_CAUSE_BEARERCAPABILITY_NOTAVAIL = 58
AST_CAUSE_BEARERCAPABILITY_NOTIMPL = 65
AST_CAUSE_CHAN_NOT_IMPLEMENTED = 66
AST_CAUSE_FACILITY_NOT_IMPLEMENTED = 69
AST_CAUSE_INVALID_CALL_REFERENCE = 81
AST_CAUSE_INCOMPATIBLE_DESTINATION = 88
AST_CAUSE_INVALID_MSG_UNSPECIFIED = 95
AST_CAUSE_MANDATORY_IE_MISSING = 96
AST_CAUSE_MESSAGE_TYPE_NONEXIST = 97
AST_CAUSE_WRONG_MESSAGE = 98
AST_CAUSE_IE_NONEXIST = 99
AST_CAUSE_INVALID_IE_CONTENTS = 100
AST_CAUSE_WRONG_CALL_STATE = 101
AST_CAUSE_RECOVERY_ON_TIMER_EXPIRE = 102
AST_CAUSE_PROTOCOL_ERROR = 111
AST_CAUSE_INTERWORKING = 127

AST_STATE_DOWN = 0
AST_STATE_RESERVED = 1
AST_STATE_OFFHOOK = 2
AST_STATE_DIALING = 3
AST_STATE_RING = 4
AST_STATE_RINGING = 5
AST_STATE_UP = 6
AST_STATE_BUSY = 7
AST_STATE_DIALING_OFFHOOK = 8
AST_STATE_PRERING = 9



from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase


class TestBlindXfer(ChannelEventsTestCase):

    def test_xfer_blind_abbcac(self):
        """
        Test a blind transfer where B initiates the transfer.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_abbcac.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='SIP/150010001-00000003',
            uniqueid='07b796be1962-1529998959.65',
            linkedid='07b796be1962-1529998959.65',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
            state=6,
        )

        a_chan_transferred = a_chan.replace(
            account_code='150010002',
            caller_id=a_chan.caller_id.replace(name=''),
            exten='203',
        )

        b_chan = SimpleChannel(
            name='SIP/150010002-00000004',
            uniqueid='07b796be1962-1529998959.73',
            linkedid='07b796be1962-1529998959.65',
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        c_chan = SimpleChannel(
            name='SIP/150010003-00000005',
            uniqueid='07b796be1962-1529998966.104',
            linkedid='07b796be1962-1529998959.65',
            account_code='150010001',
            caller_id=CallerId(num='203'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='201'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_blind_transfer', {
                'caller': a_chan_transferred,
                'targets': [c_chan.replace(state=5)],
                'transferer': b_chan.replace(exten='203'),
            }),
            ('on_up', {
                'caller': a_chan_transferred,
                'target': c_chan,
            }),
            ('on_hangup', {
                'caller': a_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_xfer_blind_abacbc(self):
        """
        Test a blind transfer where A initiates the transfer.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_abacbc.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='SIP/150010001-00000006',
            uniqueid='07b796be1962-1529998985.130',
            linkedid='07b796be1962-1529998985.130',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
            state=6,
        )

        b_chan = SimpleChannel(
            name='SIP/150010002-00000007',
            uniqueid='07b796be1962-1529998985.138',
            linkedid='07b796be1962-1529998985.130',
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        b_chan_transferred = b_chan.replace(
            exten='203',
        )

        c_chan = SimpleChannel(
            name='SIP/150010003-00000008',
            uniqueid='07b796be1962-1529998993.169',
            linkedid='07b796be1962-1529998985.130',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='203'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='202'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_blind_transfer', {
                'caller': b_chan_transferred,
                'targets': [c_chan.replace(state=5)],
                'transferer': a_chan.replace(exten='203'),
            }),
            ('on_up', {
                'caller': b_chan_transferred,
                'target': c_chan,
            }),
            ('on_hangup', {
                'caller': b_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_xfer_blind_a_no_answer(self):
        """
        Test a blind transfer from A where the transfer target is down.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_a_no_answer.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000012',
                'targets': ['SIP/150010002-00000013'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000012',
                'target': 'SIP/150010002-00000013',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-00000012',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_xfer_blind_b_no_answer(self):
        """
        Test a blind transfer from B where the transfer target is down.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_b_no_answer.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000015',
                'targets': ['SIP/150010002-00000016'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000015',
                'target': 'SIP/150010002-00000016',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-00000015',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_xfer_blind_reject(self):
        """
        Test a blind transfer from where the target rejects the transfer.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_reject.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000018',
                'targets': ['SIP/150010002-00000019'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000018',
                'target': 'SIP/150010002-00000019',
            }),
            ('on_blind_transfer', {
                'caller': 'SIP/150010001-00000018',
                'targets': ['SIP/150010003-0000001a'],
                'transferer': 'SIP/150010002-00000019',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-00000018',
                'reason': 'no-answer',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_xfer_blind_group(self):
        """
        Test a blind transfer where the call is transferred to a group.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_group.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000017',
                'targets': ['SIP/150010001-00000018'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-00000017',
                'target': 'SIP/150010001-00000018',
            }),
            ('on_blind_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-00000017',
                'targets': [
                    'SIP/150010002-00000019',
                    'SIP/150010003-0000001a',
                ],
                'transferer': 'SIP/150010001-00000018',
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-00000017',
                'target': 'SIP/150010003-0000001a',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000017',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_xfer_blind_group_no_answer(self):
        """
        Test a blind transfer where the call is transferred to a group.
        """
        events_file = 'fixtures/xfer_blind/xfer_blind_group_no_answer.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000023',
                'targets': ['SIP/150010001-00000024'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-00000023',
                'target': 'SIP/150010001-00000024',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000023',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)



from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase


class TestMiscXfer(ChannelEventsTestCase):
    """
    Test various types of esoteric call types where the participants change.

    Currently, this class tests call pickups and call forwarding.
    """

    def test_sip_pickup(self):
        """
        Test call pickup performed with a SIP Replaces INVITE.
        """
        events = self.run_and_get_events('fixtures/xfer_misc/sip_pickup.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-0000001f',
                'targets': ['SIP/150010002-00000020'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-0000001f',
                'target': 'SIP/150010003-00000021',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-0000001f',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_star_pickup(self):
        """
        Test call pickup performed with the Pickup app in Asterisk.
        """
        events = self.run_and_get_events('fixtures/xfer_misc/star_pickup.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000022',
                'targets': ['SIP/150010002-00000023'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000022',
                'target': 'SIP/150010003-00000024',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010001-00000022',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

    def test_call_forwarding(self):
        """
        Test a call where the call is locally forwarded to another phone.
        """
        events = self.run_and_get_events(
            'fixtures/xfer_misc/call_forwarding.json')

        calling_chan = SimpleChannel(
            name='SIP/voipgrid-siproute-docker-00000016',
            uniqueid='b6093874285e-1530191379.305',
            linkedid='b6093874285e-1530191379.305',
            account_code='15001',
            caller_id=CallerId(num='+31260010001'),
            cid_calling_pres=None,
            connected_line=CallerId(),
            exten='+31150010001',
            state=6,
        )

        target_chan = SimpleChannel(
            name='SIP/150010002-00000018',
            uniqueid='b6093874285e-1530191380.328',
            linkedid='b6093874285e-1530191379.305',
            account_code='15001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(num='+31260010001'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': calling_chan.replace(state=4),
                'targets': [target_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': calling_chan,
                'target': target_chan,
            }),
            ('on_hangup', {
                'caller': calling_chan,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_call_forwarding_to_group(self):
        """
        Test call forwarding where the call is forwarded to a group.
        """
        events = self.run_and_get_events(
            'fixtures/xfer_misc/call_forwarding_to_group.json')

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000021',
                'targets': [
                    'SIP/150010002-00000023',
                ],
            }),
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000021',
                'targets': [
                    'SIP/150010003-00000024',
                ],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-00000021',
                'target': 'SIP/150010003-00000024',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000021',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)



class MissingBridgeUniqueid(KeyError):
    pass


class Bridge(object):
    """
    The Bridge object represents a Bridge in Asterisk.

    With Asterisk 12+, Asterisk creates bridges and puts channels into them to
    make audio flow between the channels. This class is a Python representation
    of such bridges.
    """

    def __init__(self, event):
        """
        Create a new bridge object.

        Args:
            event (Event): A BridgeCreate event.
        """
        self.uniqueid = event['BridgeUniqueid']
        self.type = event['BridgeType']
        self.technology = event['BridgeTechnology']
        self.creator = event['BridgeCreator']
        self.video_source_mode = event['BridgeVideoSourceMode']

        self.peers = set()

    def __len__(self):
        """
        Get the number of channels in this bridge.

        Returns:
            int: The number of channels in this bridge.
        """
        return len(self.peers)

    def __repr__(self):
        """
        Get a textual representation of this bridge.

        Returns:
            str: A representation of this bridge.
        """
        return '<Bridge(id={self.uniqueid},peers={peers})>'.format(
            self=self,
            peers=','.join([chan.name for chan in self.peers]),
        )


class BridgeDict(dict):
    """
    A dict which raises a MissingBridgeUniqueid exception if a key is missing.
    """

    def __getitem__(self, item):
        try:
            return super(BridgeDict, self).__getitem__(item)
        except KeyError:
            raise MissingBridgeUniqueid(item)



import json
import os
from unittest import TestCase


class BaseTestCase(TestCase):
    """
    BaseTestCase that has methods to open and parse json event logs.
    """

    def open_file(self, filename, *args, **kwargs):
        """"
        Open a filename. Aditional arguments are passed to open().

        Args:
            filename (str): File to open (relative to current path).
        """
        path = os.path.dirname(__file__)
        filename = os.path.join(path, filename)
        return open(filename, *args, **kwargs)

    def load_events_from_disk(self, filename):
        """
        Load events from a json event log.

        Args:
            filename (str): File to open (relative to current path).

        Returns:
            list: list of events
        """
        with self.open_file(filename, 'r') as f:
            events = json.load(f)
        return events



from unittest import TestCase
from unittest.mock import MagicMock

from cacofonisk import BaseReporter
from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from cacofonisk.reporters import MultiReporter


class MultiReporterTestCase(TestCase):

    def setUp(self):
        self.mock_reporter = MagicMock(spec=BaseReporter)
        self.multi_reporter = MultiReporter([self.mock_reporter])

        self.a_party = SimpleChannel(
            name='SIP/voipgrid-siproute-docker-00000036',
            uniqueid='f29ea68048f6-1530017224.892',
            linkedid='f29ea68048f6-1530017224.892',
            account_code='15001',
            caller_id=CallerId(num='+31150010001'),
            cid_calling_pres=None,
            connected_line=CallerId(),
            exten='+31150010004',
            state=6,
        )

        self.b_party = SimpleChannel(
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
            linkedid='195176c06ab8-1529936170.42',
            name='SIP/150010001-00000004',
            state=6,
            uniqueid='195176c06ab8-1529936170.42',
        )

        self.c_party = SimpleChannel(
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            linkedid='195176c06ab8-1529936170.42',
            name='SIP/150010002-00000005',
            state=6,
            uniqueid='195176c06ab8-1529936170.50',
        )

    def test_close(self):
        self.multi_reporter.close()
        self.mock_reporter.close.assert_called_once_with()

    def test_on_event(self):
        event_dict = {'Linkedid': '195176c06ab8-1529936170.42'}
        self.multi_reporter.on_event(event_dict)
        self.mock_reporter.on_event.assert_called_once_with(event_dict)

    def test_on_user_event(self):
        event_dict = {'Linkedid': '195176c06ab8-1529936170.42'}
        self.multi_reporter.on_user_event(self.a_party, event_dict)
        self.mock_reporter.on_user_event.assert_called_once_with(self.a_party, event_dict)

    def test_on_b_dial(self):
        targets = [self.b_party, self.c_party]

        self.multi_reporter.on_b_dial(self.a_party, targets)
        self.mock_reporter.on_b_dial.assert_called_once_with(
            self.a_party, targets)

    def test_on_up(self):
        self.multi_reporter.on_up(self.a_party, self.b_party)
        self.mock_reporter.on_up.assert_called_once_with(
            self.a_party, self.b_party)

    def test_on_attended_transfer(self):
        self.multi_reporter.on_attended_transfer(
            self.a_party, self.b_party, self.c_party)
        self.mock_reporter.on_attended_transfer.assert_called_once_with(
            self.a_party, self.b_party, self.c_party)

    def test_on_blonde_transfer(self):
        self.multi_reporter.on_blonde_transfer(
            self.a_party, self.b_party, [self.c_party])
        self.mock_reporter.on_blonde_transfer.assert_called_once_with(
            self.a_party, self.b_party, [self.c_party])

    def test_on_blind_transfer(self):
        self.multi_reporter.on_blind_transfer(
            self.a_party, self.b_party, [self.c_party])
        self.mock_reporter.on_blind_transfer.assert_called_once_with(
            self.a_party, self.b_party, [self.c_party])

    def test_on_hangup(self):
        self.multi_reporter.on_hangup(self.a_party, 'busy')
        self.mock_reporter.on_hangup.assert_called_once_with(
            self.a_party, 'busy')



from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase


class TestAttnXfer(ChannelEventsTestCase):
    def test_xfer_abacbc(self):
        """
        Test attended transfer where A transfers B to C.

        First of all, we need to get notifications that calls are being
        made:
        - 201 (126680001) calls 202
        - 201 calls 203 (126680003)

        Secondly, we need notifications that an (attended) transfer has
        happened:
        - 201 joins the other channels (202 <--> 203)
        """
        events_file = 'fixtures/xfer_attended/xfer_abacbc.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='SIP/150010001-00000028',
            uniqueid='195176c06ab8-1529941216.590',
            linkedid='195176c06ab8-1529941216.590',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
            state=6,
        )

        a_chan_3pcc = a_chan.replace(
            name='SIP/150010001-0000002a',
            uniqueid='195176c06ab8-1529941225.617',
            linkedid='195176c06ab8-1529941225.617',
            exten='203',
        )

        b_chan = SimpleChannel(
            name='SIP/150010002-00000029',
            uniqueid='195176c06ab8-1529941216.598',
            linkedid='195176c06ab8-1529941216.590',
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        b_chan_transferred = b_chan.replace(exten='203')

        c_chan = SimpleChannel(
            name='SIP/150010003-0000002b',
            uniqueid='195176c06ab8-1529941225.625',
            linkedid='195176c06ab8-1529941225.617',
            account_code='150010001',
            caller_id=CallerId(num='203'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_b_dial', {
                'caller': a_chan_3pcc.replace(state=4),
                'targets': [c_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan_3pcc,
                'target': c_chan,
            }),
            ('on_attended_transfer', {
                'caller': b_chan_transferred,
                'target': c_chan,
                'transferer': a_chan_3pcc,
            }),
            ('on_hangup', {
                'caller': b_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_xfer_abbcac(self):
        """
        Test attended transfer where B transfers A to C.

        First of all, we need to get notifications that calls are being
        made:
        - +31501234567 calls 201 (126680001)
        - 201 calls 202

        Secondly, we need notifications that an (attended) transfer has
        happened:
        - 201 joins the other channels (+31501234567 <--> 202)
        """
        events_file = 'fixtures/xfer_attended/xfer_abbcac.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='SIP/150010001-0000002c',
            uniqueid='195176c06ab8-1529941338.663',
            linkedid='195176c06ab8-1529941338.663',
            account_code='150010001',
            caller_id=CallerId(name='Andrew Garza', num='201'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='202',
            state=6,
        )

        a_chan_transferred = a_chan.replace(exten='203')

        b_chan = SimpleChannel(
            name='SIP/150010002-0000002d',
            uniqueid='195176c06ab8-1529941338.671',
            linkedid='195176c06ab8-1529941338.663',
            account_code='150010001',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        b_chan_3pcc = b_chan.replace(
            name='SIP/150010002-0000002e',
            uniqueid='195176c06ab8-1529941342.690',
            linkedid='195176c06ab8-1529941342.690',
            account_code='150010002',
            caller_id=CallerId(name='Christina Arroyo', num='202'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(),
            exten='203',
        )

        c_chan = SimpleChannel(
            name='SIP/150010003-0000002f',
            uniqueid='195176c06ab8-1529941343.698',
            linkedid='195176c06ab8-1529941342.690',
            account_code='150010002',
            caller_id=CallerId(name='', num='203'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Christina Arroyo', num='202'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_b_dial', {
                'caller': b_chan_3pcc.replace(state=4),
                'targets': [c_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': b_chan_3pcc,
                'target': c_chan,
            }),
            ('on_attended_transfer', {
                'caller': a_chan_transferred,
                'target': c_chan,
                'transferer': b_chan_3pcc,
            }),
            ('on_hangup', {
                'caller': a_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)

    def test_xfer_abbcac_anonymous(self):
        """
        Test that an attended transfer where the caller is anonymous works.
        """
        events_file = 'fixtures/xfer_attended/xfer_abbcac_anon.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/voipgrid-siproute-docker-00000027',
                'targets': ['SIP/150010001-00000028'],
            }),
            ('on_up', {
                'caller': 'SIP/voipgrid-siproute-docker-00000027',
                'target': 'SIP/150010001-00000028',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010001-00000029',
                'targets': ['SIP/150010003-0000002a'],
            }),
            ('on_up', {
                'caller': 'SIP/150010001-00000029',
                'target': 'SIP/150010003-0000002a',
            }),
            ('on_attended_transfer', {
                'caller': 'SIP/voipgrid-siproute-docker-00000027',
                'target': 'SIP/150010003-0000002a',
                'transferer': 'SIP/150010001-00000029',
            }),
            ('on_hangup', {
                'caller': 'SIP/voipgrid-siproute-docker-00000027',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

        # Check the privacy flag was set correctly before and after the
        # transfer.
        self.assertEqual(
            '33 (Presentation Prohibited, Passed Screen)',
            events[0][1]['caller'].cid_calling_pres,
        )

        self.assertEqual(
            '33 (Presentation Prohibited, Passed Screen)',
            events[4][1]['caller'].cid_calling_pres,
        )

    def test_xfer_abcbac(self):
        """
        Test attn transfer where A and C call B, and B transfers them together.
        """
        events_file = 'fixtures/xfer_attended/xfer_abcbac.json'
        events = self.run_and_get_events(events_file)

        expected_events = [
            ('on_b_dial', {
                'caller': 'SIP/150010002-0000003c',
                'targets': ['SIP/150010001-0000003d'],
            }),
            ('on_up', {
                'caller': 'SIP/150010002-0000003c',
                'target': 'SIP/150010001-0000003d',
            }),
            ('on_b_dial', {
                'caller': 'SIP/150010003-0000003e',
                'targets': ['SIP/150010001-0000003f'],
            }),
            ('on_up', {
                'caller': 'SIP/150010003-0000003e',
                'target': 'SIP/150010001-0000003f',
            }),
            ('on_attended_transfer', {
                'caller': 'SIP/150010003-0000003e',
                'target': 'SIP/150010002-0000003c',
                'transferer': 'SIP/150010001-0000003d',
            }),
            ('on_hangup', {
                'caller': 'SIP/150010003-0000003e',
                'reason': 'completed',
            }),
        ]

        self.assertEqualChannels(expected_events, events)

        # Check the transfer extensions.
        self.assertEqual(
            events[4][1]['target'].caller_id.num,
            events[4][1]['caller'].exten,
        )

        self.assertEqual(
            events[4][1]['caller'].exten,
            events[5][1]['caller'].exten,
        )



"""
Implement a Runner which reads files, instatiates a EventHandler and calls
the EventHandler accordingly.

If the main application desires to replay previously stored events (an
event replay log), the FileRunner is the runner to use.

Events are loaded from a ``.json`` file which holds a list of
dictionaries.
"""
from json import load

from ..handlers import EventHandler


class FileRunner(object):
    def __init__(self, files, reporter, channel_manager_class=EventHandler):
        """
        FileRunner is a Runner that reads from one or more files.

        Args:
            files (list): A list of strings containing filenames or,
            a string containing a filename.
            reporter (Reporter): The reporter to use for this Runner.
            channel_manager_class: The EventHandler to instantiate for this
                Runner.
        """
        if type(files) == str:
            self.files = [files]
        elif type(files) == list:
            self.files = files
        else:
            raise TypeError('Expected string or list for files argument')
        self.reporter = reporter
        self.channel_manager_class = channel_manager_class
        self.channel_managers = []

    def _load_events_from_disk(self, filename):
        """
        Read the file with the given file name and return the JSON contents.

        Args:
            filename (str): The name of the file to read.

        Returns:
            A JSON object.
        """
        with open(filename, 'r') as f:
            events = load(f)
        return events

    def run(self):
        """
        Read all the events from the files and pass them to channel_manager.
        """
        for filename in self.files:
            events = self._load_events_from_disk(filename)
            channel_manager = self.channel_manager_class(
                reporter=self.reporter)
            interesting_events = channel_manager.event_handlers().keys()

            for event in events:
                if (
                        not channel_manager.FILTER_EVENTS or
                        event['Event'] in interesting_events
                   ):
                    channel_manager.on_event(event)

            self.channel_managers.append(channel_manager)



from cacofonisk.callerid import CallerId
from cacofonisk.channel import SimpleChannel
from tests.replaytest import ChannelEventsTestCase


class TestAttnXfer(ChannelEventsTestCase):
    
    def test_xfer_abbc(self):
        """
        Test threeway transfer where B merges with A and C.

        First of all, we need to get notifications that calls are being
        made:
        - 203 (150010003) calls 202 (150010002)
        - 202 calls 204 (150010004)

        Secondly, we need notifications that an threeway transfer has
        happened:
        - 202 joins the channels (202, 204) into a single brigde
        """
        self.maxDiff = None
        events_file = 'fixtures/xfer_threeway/xfer_threeway_abbc.json'
        events = self.run_and_get_events(events_file)

        a_chan = SimpleChannel(
            name='PJSIP/150010003-00000003',
            uniqueid='ac670c3c69a6-1698413067.111',
            linkedid='ac670c3c69a6-1698413067.111',
            account_code='150010003',
            caller_id=CallerId(name='Megan Estes', num='203'),
            cid_calling_pres='1 (Presentation Allowed, Passed Screen)',
            connected_line=CallerId(num="202"),
            exten='202',
            state=6,
        )

        a_chan_3pcc = a_chan.replace(
            name='SIP/150010001-0000002a',
            uniqueid='195176c06ab8-1529941225.617',
            linkedid='195176c06ab8-1529941225.617',
            exten='203',
        )

        b_chan = SimpleChannel(
            name='PJSIP/150010002-00000004',
            uniqueid='ac670c3c69a6-1698413067.122',
            linkedid='ac670c3c69a6-1698413067.111',
            account_code='150010003',
            caller_id=CallerId(num='202'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Megan Estes', num='203'),
            exten='s',
            state=6,
        )

        b_chan_transferred = b_chan.replace(exten='203')

        c_chan = SimpleChannel(
            name='SIP/150010003-0000002b',
            uniqueid='195176c06ab8-1529941225.625',
            linkedid='195176c06ab8-1529941225.617',
            account_code='150010001',
            caller_id=CallerId(num='203'),
            cid_calling_pres='0 (Presentation Allowed, Not Screened)',
            connected_line=CallerId(name='Andrew Garza', num='201'),
            exten='s',
            state=6,
        )

        expected_events = [
            ('on_b_dial', {
                'caller': a_chan.replace(state=4),
                'targets': [b_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan,
                'target': b_chan,
            }),
            ('on_b_dial', {
                'caller': a_chan.replace(state=4), # ?
                'targets': [c_chan.replace(state=5)],
            }),
            ('on_up', {
                'caller': a_chan_3pcc,
                'target': c_chan,
            }),
            ('on_threeway_transfer', {
                'caller': b_chan_transferred,
                'target': c_chan,
                'transferer': a_chan_3pcc,
            }),
            ('on_hangup', {
                'caller': b_chan_transferred,
                'reason': 'completed',
            }),
            ('on_hangup', {
                'caller': b_chan_transferred,
                'reason': 'completed',
            }),
        ]

        self.assertEqual(expected_events, events)



import glob
import logging
import os

from tests.replaytest import BogoRunner, ChannelEventsTestCase, TestReporter


class TestPartial(ChannelEventsTestCase):

    def setUp(self):
        """
        Disable log output.

        Disable log output because missing channels/bridges create A LOT of
        log output.
        """
        logging.disable(logging.WARN)

    def tearDown(self):
        """
        Re-enable logging.
        """
        logging.disable(logging.NOTSET)

    def test_partial_events(self):
        """
        Test cacofonisk doesn't crash if it's attached to running Asterisk.
        """
        files = glob.glob(os.path.join(
            os.path.dirname(__file__),
            'fixtures', '**', '*.json'
        ))

        for filename in files:
            events = self.load_events_from_disk(filename)
            events = list(filter(lambda e: e['Event'] not in ('VarSet', 'Newexten'), events))
            while len(events) != 0:
                events.pop(0)
                reporter = TestReporter()
                runner = BogoRunner(events=events, reporter=reporter)
                runner.run()
                assert len(runner.channel_managers) == 1
